#include <qapplication.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qtabbar.h>
#define INCLUDE_MENUITEM_DEF
#include <qpopupmenu.h>
#include <limits.h>
#include <kglobalsettings.h>
#include "steelstyle.h"

#define QCOORDARRLEN(x) sizeof(x)/(sizeof(QCOORD)*2)

inline int clr(int c) { return (c>255)? 255: (c<0)? 0: c; }

SteelStyle::SteelStyle(): KStyle()
{
    QPalette pal = QApplication::palette();
    QColorGroup active(pal.active());
    hl2 = active.highlight();
    int hr=hl2.red(), hg=hl2.green(), hb=hl2.blue();
    hl1 = QColor(clr(hr-51), clr(hg-51), clr(hb-51));
    hl3 = QColor(clr(hr+51), clr(hg+51), clr(hb+51));
    sbbr = pmBrush(hl2, hl3, hl1);
    slbr = pmBrush(active.button(), Qt::white, active.dark());
}

QBrush *SteelStyle::pmBrush(const QColor& bg, const QColor& br, const QColor& sh)
{
    QPixmap pm(4, 4);
    pm.fill(bg);
    QPainter p;
    p.begin(&pm);
    p.setPen(br);
    p.drawPoint(0, 0);
    p.drawPoint(2, 2);
    p.setPen(sh);
    p.drawPoint(1, 1);
    p.drawPoint(3, 3);
    p.end();
    return new QBrush(Qt::white, pm);
}

void SteelStyle::polish(QPalette& pal)
{
    pal.setColor(QPalette::Disabled, QColorGroup::Highlight, hl2);
}

void SteelStyle::polish(QWidget *wdg)
{
    if(wdg->inherits("QListBox")) {
        QPalette newPalette(wdg->palette());
        newPalette.setColor(QPalette::Active, QColorGroup::Highlight, hl3);
        newPalette.setColor(QPalette::Inactive, QColorGroup::Highlight, hl3);
        newPalette.setColor(QPalette::Disabled, QColorGroup::Highlight, hl3);
        wdg->setPalette(newPalette);
    }
}

void SteelStyle::unPolish(QWidget *wdg)
{
    wdg->setPalette(QApplication::palette(), TRUE);
}

void SteelStyle::drawButton(QPainter *p, int x, int y, int w, int h,
                            const QColorGroup &g, bool sunken, const QBrush* fill)
{
    if(w < 2 || h < 2) return;
    if(sunken)
        p->fillRect(x, y, w, h, g.brush(QColorGroup::Mid));
    else
        qDrawShadePanel(p, x, y, w, h, g, false, 1,
                        fill? fill: &g.brush(QColorGroup::Button));
}

void SteelStyle::drawBevelButton(QPainter *p, int x, int y, int w, int h,
                                 const QColorGroup &g, bool sunken, const QBrush *fill)
{
    if(w < 2 || h < 2) return;
    qDrawShadePanel(p, x, y, w, h, g, true, 1, 0);
    if(sunken)
        p->fillRect(x+1, y+1, w-2, h-2, g.brush(QColorGroup::Mid));
    else
        qDrawShadePanel(p, x+1, y+1, w-2, h-2, g, false, 1,
                        fill? fill: &g.brush(QColorGroup::Button));
}

void SteelStyle::drawPanel(QPainter *p, int x, int y, int w, int h,
                           const QColorGroup &g, bool sunken,
                           int lineWidth, const QBrush *fill)
{
    QStyle::drawPanel(p, x, y, w, h, g, sunken, lineWidth, fill);
}

void SteelStyle::drawPopupPanel(QPainter *p, int x, int y, int w, int h,
                                const QColorGroup &g,
                                int lineWidth, const QBrush *fill)
{
    QStyle::drawPopupPanel(p, x, y, w, h, g, lineWidth, fill);
}

void SteelStyle::drawArrow(QPainter *p, Qt::ArrowType type, bool,
                           int x, int y, int w, int h,
                           const QColorGroup &g, bool enabled, const QBrush *fill)
{
    QPointArray a;
    switch(type) {
    case UpArrow:
        a.setPoints(8, -4,1, 3,1, -3,0, 2,0, -2,-1, 1,-1, -1,-2, 0,-2);
        break;
    case DownArrow:
        a.setPoints(8, -4,-1, 3,-1, -3,0, 2,0, -2,1, 1,1, -1,2, 0,2);
        break;
    case LeftArrow:
        a.setPoints(8, 1,-4, 1,3, 0,-3, 0,2, -1,-2, -1,1, -2,-1, -2,0);
        break;
    case RightArrow:
        a.setPoints(8, -1,-4, -1,3, 0,-3, 0,2, 1,-2, 1,1, 2,-1, 2,0);
        break;
    }
    if(a.isNull())
        return;
    if(fill) p->fillRect(x, y, w, h, *fill);
    if(enabled) {
        a.translate(x+w/2, y+h/2);
        p->setPen(g.foreground());
        p->drawLineSegments(a);
    } else {
        a.translate(x+w/2+1, y+h/2+1);
        p->setPen(g.light());
        p->drawLineSegments(a);
        a.translate(-1, -1);
        p->setPen(g.mid());
        p->drawLineSegments(a);
    }
}

QSize SteelStyle::exclusiveIndicatorSize() const
{
    return QSize(13, 13);
}
void SteelStyle::drawExclusiveIndicatorMask(QPainter *p, int x, int y, int w, int h, bool /* on */)
{
    QColorGroup g(color1, color1, color1, color1, color1, color1, color1, color1, color0);
    drawExclusiveIndicator(p , x, y, w, h, g, FALSE, FALSE, FALSE);
}

void SteelStyle::drawExclusiveIndicator(QPainter* p,  int x, int y, int w, int h, const QColorGroup& g,
                            bool on, bool down, bool enabled)
{
    static QCOORD pts1[] = {
        1,9, 1,8, 0,7, 0,4, 1,3, 1,2, 2,1, 3,1, 4,0, 7,0, 8,1, 9,1,
        10,2, 10,3, 11,4, 11,7, 10,8, 10,9, 9,10, 8,10, 7,11, 4,11,
        3,10, 2,10};
    static QCOORD pts2[] = {
        2,9, 2,8, 1,7, 1,4, 2,3, 2,2, 3,2, 4,1, 7,1, 8,2, 9,2};
    static QCOORD pts3[] = {
        10,1, 11,2, 11,3, 12,4, 12,7, 11,8, 11,9, 9,11, 8,11, 7,12, 4,12,
        3,11, 2,11};
    static QCOORD pts4[] = {
        2,9, 2,8, 1,7, 1,4, 2,3, 2,2, 3,2, 4,1, 7,1, 8,2, 9,2,
        9,3, 10,4, 10,7, 9,8, 9,9, 8,9, 7,10, 4,10, 3,9};

    p->eraseRect(x, y, w, h);
    p->setPen(g.dark());
    QPointArray a(QCOORDARRLEN(pts1), pts1);
    a.translate(x, y);
    p->drawPolyline(a);
    p->setPen(g.light());
    a.setPoints(QCOORDARRLEN(pts2), pts2);
    a.translate(x, y);
    p->drawPolyline(a);
    a.setPoints(QCOORDARRLEN(pts3), pts3);
    a.translate(x, y);
    p->drawPolyline(a);
    p->setPen(NoPen);
    p->setBrush((down || !enabled)? g.mid(): g.background());
    a.setPoints(QCOORDARRLEN(pts4), pts4);
    a.translate(x, y);
    p->drawPolygon(a);
    if(on) {
        p->setPen(NoPen);
        p->setBrush(g.text());
        p->drawRect(x+4, y+3, 4, 6);
        p->drawRect(x+3, y+4, 6, 4);
    }
}

QSize SteelStyle::indicatorSize() const
{
    return QSize(13, 13);
}

void SteelStyle::drawIndicator(QPainter* p,  int x, int y, int w, int h, const QColorGroup& g,
                               int state, bool down, bool ena)
{
    QBrush fill;
    if(state == QButton::NoChange) {
        QBrush b = p->brush();
        QColor c = p->backgroundColor();
        p->setBackgroundMode(TransparentMode);
        p->setBackgroundColor(green);
        fill = QBrush(g.base(), Dense4Pattern);
        p->setBackgroundColor(c);
        p->setBrush(b);
    } else
        fill = g.brush(QColorGroup::Button);
    drawBevelButton(p, x, y, w, h, g, down, &fill);
    if(state != QButton::Off)
        drawCheckMark(p, x+3, y+3, 7, 6, g, state == QButton::NoChange, !ena);
}

void SteelStyle::drawFocusRect(QPainter *p, const QRect &r,
                               const QColorGroup&, const QColor*, bool)
{
    p->setPen(QApplication::palette().active().highlight());
    p->drawRect(r);
}

void SteelStyle::drawComboButton(QPainter *p, int x, int y, int w, int h,
                                 const QColorGroup &g, bool sunken,
                                 bool editable, bool, const QBrush *fill)
{
    static QCOORD pts[] = {0,-2, 9,-2, 1,-1, 8,-1, 2,0, 7,0, 3,1, 6,1, 4,2, 5,2};
    QPointArray a(QCOORDARRLEN(pts), pts);
    if(!editable)
        drawBevelButton(p, x, y, w, h, g, sunken, fill? fill: &g.brush(QColorGroup::Button));
    else {
        qDrawShadePanel(p, x, y, w-h, h, g, true, 1, 0);
        qDrawShadePanel(p, x+1, y+1, w-2, h-2, g, false, 1, &g.brush(QColorGroup::Base));
        drawBevelButton(p, x+w-h, y, h, h, g, sunken, fill? fill: &g.brush(QColorGroup::Button));
    }
    p->setPen(g.foreground());
    int c = h / 2;
    a.translate(x+w-c-6, y+c);
    p->drawLineSegments(a);

}

QRect SteelStyle::comboButtonRect(int x, int y, int w, int h)
{
    return buttonRect(x+2, y+2, w-h-4, h-4);
}

QRect SteelStyle::comboButtonFocusRect(int x, int y, int w, int h)
{
    return buttonRect(x+3, y+3, w-h-6, h-6);
}

void SteelStyle::drawPushButton(QPushButton* btn, QPainter *p)
{
    QColorGroup g = btn->colorGroup();
    int x1, y1, x2, y2;

    btn->rect().coords(&x1, &y1, &x2, &y2);
    p->setPen(g.foreground());
    p->setBrush(QBrush(g.button(), NoBrush));

    QBrush fill;
    if(btn->isDown())
        fill = g.brush(QColorGroup::Mid);
    else if(btn->isOn())
        fill = QBrush(g.mid(), Dense4Pattern);
    else
        fill = g.brush(QColorGroup::Button);

    if(btn->isDefault()) { x1 += 1; y1 += 1; }
    drawBevelButton(p, x1, y1, x2-x1+1, y2-y1+1, g, btn->isOn() || btn->isDown(),
               &fill);
    if(btn->isDefault()) {
        p->setPen(g.dark());
        p->drawRect(x1-1, y1-1, x2-x1, y2-y1);
    }

    if(btn->isMenuButton()) {
        int dx = (y1-y2-4)/3;
        drawArrow(p, DownArrow, FALSE,
                  x2 - dx, dx, y1, y2 - y1,
                  g, btn->isEnabled());
    }

    if(p->brush().style() != NoBrush)
        p->setBrush(NoBrush);
}

void SteelStyle::drawPushButtonLabel(QPushButton *btn, QPainter *p)
{
    QCommonStyle::drawPushButtonLabel(btn, p);
}

void SteelStyle::getButtonShift(int &x, int &y)
{
    x = y = 0;
}

int SteelStyle::defaultFrameWidth() const
{
    return 1;
}

void SteelStyle::tabbarMetrics(const QTabBar* t, int& hframe, int& vframe, int& overlap)
{
    overlap = 0;
    hframe = 16;
    vframe = 0;
    if(t->shape() == QTabBar::RoundedAbove || t->shape() == QTabBar::RoundedBelow)
        vframe += 8;
}

void SteelStyle::drawTab(QPainter* p, const QTabBar* tb, QTab* t, bool selected)
{
    if(tb->shape() != QTabBar::RoundedAbove){
        KStyle::drawTab(p, tb, t, selected);
        return;
    }

    int x1, y1, x2, y2;
    QColorGroup g = tb->colorGroup();
    t->r.coords(&x1, &y1, &x2, &y2);
    --y2;
    QPointArray a;
    if(x1 == 0) {
        p->setPen(g.light());
        p->drawLine(0, y2+1, 1, y2+1);
    }
    a.setPoints(5,
                x1, y2, x1, y1+6,
                x1+6, y1, x2, y1,
                x2, y2
               );
    p->setPen(g.dark());
    p->drawPolyline(a);
    a.setPoints(5,
                x1, y2+1, x1+1, y2+1,
                x1+1, y1+6, x1+6, y1+1,
                x2-1, y1+1
               );
    p->setPen(selected? g.light(): g.button());
    p->drawPolyline(a);
    a.setPoints(5,
                x1+2, y2+1, x1+2, y1+6,
                x1+6, y1+2, x2, y1+2,
                x2, y2+1
               );
    p->setPen(NoPen);
    p->setBrush(g.brush(selected? QColorGroup::Button: QColorGroup::Mid));
    p->drawPolygon(a);
    if(!selected) {
        p->setPen(g.light());
        p->drawLine(x1, y2+1, x2, y2+1);
    }
}

#define HORIZONTAL      (sb->orientation() == QScrollBar::Horizontal)
#define VERTICAL        !HORIZONTAL
#define SLIDER_MIN      9

void SteelStyle::scrollBarMetrics(const QScrollBar* sb, int &sliderMin, int &sliderMax, int &sliderLength, int&buttonDim)
{
    int maxLength;
    int length = HORIZONTAL? sb->width(): sb->height();
    int extent = HORIZONTAL? sb->height(): sb->width();

    buttonDim = (length > (extent - 1)*2)? extent: (length/2 - 1);
    sliderMin = buttonDim;
    maxLength  = length - buttonDim*2;
    if(sb->maxValue() == sb->minValue()) {
        sliderLength = maxLength;
    } else {
        sliderLength = (sb->pageStep()*maxLength)/
            (sb->maxValue()-sb->minValue()+sb->pageStep());
        uint range = sb->maxValue()-sb->minValue();
        if(sliderLength < SLIDER_MIN || range > INT_MAX/2)
            sliderLength = SLIDER_MIN;
        if(sliderLength > maxLength)
            sliderLength = maxLength;
    }
    sliderMax = sliderMin + maxLength - sliderLength;
}

void SteelStyle::drawScrollBarControls(QPainter* p, const QScrollBar* sb,
                                       int sliderStart, uint controls, uint activeControl)
{
#define ADD_LINE_ACTIVE (activeControl == AddLine)
#define SUB_LINE_ACTIVE (activeControl == SubLine)
    QColorGroup g  = sb->colorGroup();

    int sliderMin, sliderMax, sliderLength, buttonDim;
    scrollBarMetrics(sb, sliderMin, sliderMax, sliderLength, buttonDim);

    if(sliderStart > sliderMax)
        sliderStart = sliderMax;

    int dimB = buttonDim;
    QRect addB;
    QRect subB;
    QRect addPageR;
    QRect subPageR;
    QRect sliderR;
    int addX, addY, subX, subY;
    int length = HORIZONTAL? sb->width(): sb->height();
    int extent = HORIZONTAL? sb->height(): sb->width();

    if(HORIZONTAL) {
        subY = addY = (extent - dimB) / 2;
        subX = 0;
        addX = length - dimB;
    } else {
        subX = addX = (extent - dimB) / 2;
        subY = 0;
        addY = length - dimB;
    }

    subB.setRect(subX,subY,dimB,dimB);
    addB.setRect(addX,addY,dimB,dimB);

    int sliderEnd = sliderStart + sliderLength;
    if(HORIZONTAL) {
        subPageR.setRect(subB.right() + 1, 0,
                         sliderStart - subB.right() - 1 , extent);
        addPageR.setRect(sliderEnd, 0, addX - sliderEnd, extent);
        sliderR .setRect(sliderStart, 0, sliderLength, extent);
    } else {
        subPageR.setRect(0, subB.bottom() + 1, extent,
                         sliderStart - subB.bottom() - 1);
        addPageR.setRect(0, sliderEnd, extent, addY - sliderEnd);
        sliderR .setRect(0, sliderStart, extent, sliderLength);
    }

    bool maxedOut = (sb->maxValue() == sb->minValue());
    QPointArray a(4);
    int x, y, w, h;
    p->setBrush(g.brush(QColorGroup::Button));
    if(controls & AddLine) {
        x = addB.x(), y = addB.y(), w = addB.width()-1, h = addB.height()-1;
        if(HORIZONTAL) ++y; else ++x;
        drawButton(p, x, y, w, h, g, ADD_LINE_ACTIVE);
        drawArrow(p, VERTICAL? DownArrow: RightArrow,
                  ADD_LINE_ACTIVE, x, y, w, h, g, !maxedOut);
        a.setPoint(0, addB.x(), addB.y());
        if(VERTICAL) a.setPoint(1, addB.x(), addB.bottom());
        else a.setPoint(1, addB.right(), addB.y());
        a.setPoint(2, addB.right(), addB.bottom());
        p->setPen(g.dark());
        p->drawPolyline(a, 0, 3);
    }
    if(controls & SubLine) {
        x = subB.x()+1, y = subB.y()+1, w = subB.width()-1, h = subB.height()-1;
        drawButton(p, x, y, w, h, g, SUB_LINE_ACTIVE);
        drawArrow(p, VERTICAL? UpArrow: LeftArrow,
                   SUB_LINE_ACTIVE, x, y, w, h, g, !maxedOut);
        a.setPoint(0, subB.x(), subB.bottom());
        a.setPoint(1, subB.x(), subB.y());
        a.setPoint(2, subB.right(), subB.y());
        p->setPen(g.dark());
        p->drawPolyline(a, 0, 3);
    }
    p->setPen(g.dark());
    if(maxedOut) {
        p->drawRect(sliderR);
    } else {
        if((controls & SubPage)) {
            p->fillRect(subPageR, g.brush(QColorGroup::Button));
            if(HORIZONTAL) {
                p->drawLine(subPageR.x(), subPageR.y(), subPageR.right(), subPageR.y());
                p->drawLine(subPageR.x(), subPageR.bottom(), subPageR.right(), subPageR.bottom());
                a.setPoint(0, subPageR.right(), subPageR.y()+1);
                a.setPoint(1, subPageR.x(), subPageR.y()+1);
                a.setPoint(2, subPageR.x(), subPageR.bottom()-1);
            } else {
                p->drawLine(subPageR.x(), subPageR.y(), subPageR.x(), subPageR.bottom());
                p->drawLine(subPageR.right(), subPageR.y(), subPageR.right(), subPageR.bottom());
                a.setPoint(0, subPageR.right()-1, subPageR.y());
                a.setPoint(1, subPageR.x()+1, subPageR.y());
                a.setPoint(2, subPageR.x()+1, subPageR.bottom());
            }
            if(subPageR.width() > 2 && subPageR.height() > 2) {
                p->setPen(g.mid());
                p->drawPolyline(a, 0, 3);
                p->setPen(g.dark());
            }
        }
        if((controls & AddPage)) {
            p->fillRect(addPageR, g.brush(QColorGroup::Button));
            if(HORIZONTAL) {
                p->drawLine(addPageR.x(), addPageR.y(), addPageR.right(), addPageR.y());
                p->drawLine(addPageR.x(), addPageR.bottom(), addPageR.right(), addPageR.bottom());
                a.setPoint(0, addPageR.right(), addPageR.y()+1);
                a.setPoint(1, addPageR.x(), addPageR.y()+1);
                a.setPoint(2, addPageR.x(), addPageR.bottom()-1);
            } else {
                p->drawLine(addPageR.x(), addPageR.y(), addPageR.x(), addPageR.bottom());
                p->drawLine(addPageR.right(), addPageR.y(), addPageR.right(), addPageR.bottom());
                a.setPoint(0, addPageR.right()-1, addPageR.y());
                a.setPoint(1, addPageR.x()+1, addPageR.y());
                a.setPoint(2, addPageR.x()+1, addPageR.bottom());
            }
            if(addPageR.width() > 2 && addPageR.height() > 2) {
                p->setPen(g.mid());
                p->drawPolyline(a, 0, 3);
                p->setPen(g.dark());
            }
        }
        if(controls & Slider) {
            p->drawLine(sliderR.right(), sliderR.y(), sliderR.right(), sliderR.bottom());
            a.setPoint(0, sliderR.right()-1, sliderR.y());
            a.setPoint(1, sliderR.x(), sliderR.y());
            a.setPoint(2, sliderR.x(), sliderR.bottom());
            a.setPoint(3, sliderR.right()-1, sliderR.bottom());
            p->setPen(hl1);
            p->drawPolyline(a);
            a.setPoint(0, sliderR.right()-1, sliderR.y()+1);
            a.setPoint(1, sliderR.x()+1, sliderR.y()+1);
            a.setPoint(2, sliderR.x()+1, sliderR.bottom()-1);
            p->setPen(hl3);
            p->drawPolyline(a, 0, 3);
            p->fillRect(sliderR.x()+2, sliderR.y()+2,
                        sliderR.width()-3, sliderR.height()-3, hl2);
            x = sliderR.x()+3; y = sliderR.y()+3;
            w = sliderR.width()-6; h = sliderR.height()-6;
            if(HORIZONTAL) { ++x; --w; } else { ++y; --h; }
            if(w % 2) --w;
            if(h % 2) --h;
            p->setBrushOrigin(x, y);
            p->fillRect(x, y, w, h, *sbbr);
        }
    }
}

QStyle::ScrollControl SteelStyle::scrollBarPointOver(const QScrollBar* sb, int sliderStart, const QPoint& p)
{
    return QCommonStyle::scrollBarPointOver(sb, sliderStart, p);
}

int SteelStyle::sliderLength() const
{
    return 15;
}

void SteelStyle::drawSlider(QPainter *p, int x, int y, int, int, const QColorGroup &g,
                            Orientation orient, bool, bool)
{
    static QCOORD
        hpts[] = {1,0, 13,0, 14,1, 14,8, 7,15, 0,8, 0,1, 1,0},
        vpts[] = {1,0, 8,0, 15,7, 8,14, 1,14, 0,13, 0,1, 1,0};
    QPointArray a;
    if(orient == Horizontal) a.setPoints(QCOORDARRLEN(hpts), hpts);
    else a.setPoints(QCOORDARRLEN(vpts), vpts);
    a.translate(x, y);
    p->setPen(g.foreground());
    p->setBrush(g.brush(QColorGroup::Button));
    p->drawPolygon(a);
    p->setBrushOrigin(x+2, y+2);
    p->setPen(g.light());
    if(orient == Horizontal) {
        p->drawLine(x+1, y+8, x+1, y+1);
        p->drawLine(x+1, y+1, x+13, y+1);
        p->fillRect(x+2, y+2, 10, 6, *slbr);
    } else {
        p->drawLine(x+1, y+13, x+1, y+1);
        p->drawLine(x+1, y+1, x+8, y+1);
        p->fillRect(x+2, y+2, 6, 10, *slbr);
    }
}

void SteelStyle::drawSliderGroove(QPainter *p, int x, int y, int w, int h,
                                  const QColorGroup& g, QCOORD c, Orientation orient)
{
    p->fillRect(x, y, w, h, g.brush(QColorGroup::Background));
    if(orient == Horizontal) {
        y = y+c-4;
        h = 7;
    }
    else {
        x = x+c-4;
        w = 7;
    }
    qDrawShadePanel(p, x, y, w, h, g, true);
    p->setPen(g.mid());
    p->drawLine(x+1, y+1, x+w-2, y+1);
    p->drawLine(x+1, y+1, x+1, y+h-2);

    p->setPen(g.dark());
    p->drawLine(x+1, y+h-2, x+w-2, y+h-2);
    p->drawLine(x+w-2, y+1, x+w-2, y+h-2);
}

int SteelStyle::splitterWidth() const
{
    return 6;
}

void SteelStyle::drawSplitter(QPainter *p, int x, int y, int w, int h,
                              const QColorGroup &g,  Orientation)
{
    qDrawShadePanel(p, x, y, w, h, g);
}

void SteelStyle::drawCheckMark(QPainter *p, int x, int y, int, int,
                               const QColorGroup &g, bool act, bool)
{
    static QCOORD pts[] = {0,1, 0,5, 1,1, 1,5, 2,5, 6,1, 1,4, 6,0, 2,4, 3,3};
    QPointArray a(QCOORDARRLEN(pts), pts);
    a.translate(x, y);
    p->setPen(act? g.dark(): g.foreground());
    p->drawLineSegments(a);
}

static const int steelItemFrame         = 1;
static const int steelSepHeight         = 2;
static const int steelItemHMargin       = 3;
static const int steelItemVMargin       = 1;
static const int steelArrowHMargin      = 6;
static const int steelRightBorder       = 12;
static const int steelTabSpacing        = 12;
static const int steelCheckMarkHMargin  = 2;
static const int steelCheckMarkWidth    = 12;

void SteelStyle::polishPopupMenu(QPopupMenu* p)
{
    KStyle::polishPopupMenu(p);
    p->setMouseTracking(TRUE);
    if(!p->testWState(WState_Polished))
        p->setCheckable(TRUE);
    p->setLineWidth(1);
}

int SteelStyle::extraPopupMenuItemWidth(bool checkable, int maxpmw, QMenuItem* mi, const QFontMetrics&)
{
    int w = 2*steelItemHMargin + 2*steelItemFrame;
    if(mi->pixmap())
        w += mi->pixmap()->width();
    if(!mi->text().isNull()) {
        if(mi->text().find('\t') >= 0)
            w += steelTabSpacing;
    }
    if(maxpmw)
        w += maxpmw + 6;
    if(checkable && maxpmw < steelCheckMarkWidth)
        w += steelCheckMarkWidth - maxpmw;
    if(maxpmw > 0 || checkable)
        w += steelCheckMarkHMargin;
    return w;
}

int SteelStyle::popupMenuItemHeight(bool, QMenuItem* mi, const QFontMetrics& fm)
{
    int h = 0;
    if(mi->isSeparator())
        h = steelSepHeight;
    else if(mi->pixmap())
        h = mi->pixmap()->height() + 2*steelItemFrame;
    else
        h = fm.height() + 2*steelItemVMargin + 2*steelItemFrame;
    if(!mi->isSeparator() && mi->iconSet() != 0) {
        h = QMAX(h, mi->iconSet()->pixmap(QIconSet::Small, QIconSet::Normal).height() + 2*steelItemFrame);
        int h2 = fm.height() + 2*steelItemVMargin + 2*steelItemFrame;
        if(h2 > h)
            h = h2;
    }
    return h;
}

void SteelStyle::drawPopupMenuItem(QPainter* p, bool checkable, int maxpmw, int tab, QMenuItem* mi,
                                   const QPalette& pal,
                                   bool act, bool enabled, int x, int y, int w, int h)
{
    bool dis = !enabled;
    QColorGroup g = dis? pal.disabled(): act? pal.active(): pal.inactive();

    if(checkable)
        maxpmw = QMAX(maxpmw, 12);
    int checkcol = maxpmw;


    if(mi->isSeparator()) {
        p->setPen(g.dark());
        p->drawLine(x, y, x+w, y);
        p->setPen(g.light());
        p->drawLine(x, y+1, x+w, y+1);
        return;
    }

    if(!act) {
        QBrush fill = g.brush(QColorGroup::Button);
        p->fillRect(x, y, w, h, fill);
    } else {
        QBrush fill = g.brush(QColorGroup::Highlight);
        qDrawShadePanel(p, x, y, w, h, g, true, 1, &fill);
    }

    if(mi->iconSet()) {
        QIconSet::Mode mode = dis?QIconSet::Disabled:QIconSet::Normal;
        if(act && !dis)
            mode = QIconSet::Active;
        QPixmap pixmap = mi->iconSet()->pixmap(QIconSet::Small, mode);
        int pixw = pixmap.width();
        int pixh = pixmap.height();
        QRect cr(x, y, checkcol, h);
        QRect pmr(0, 0, pixw, pixh);
        pmr.moveCenter(cr.center());
        p->setPen(g.text());
        p->drawPixmap(pmr.topLeft(), pixmap);
    } else if(checkable) {
        int mw = checkcol + steelItemFrame;
        int mh = h - 2*steelItemFrame;
        if(mi->isChecked()) {
            drawCheckMark(p, x+steelItemFrame+2,
                           y+steelItemFrame+steelItemVMargin+6, mw, mh, g, act, dis);
        }
    }

    p->setPen(act? g.highlightedText(): g.buttonText());

    QColor discol;
    if(dis) {
        discol = g.text();
        p->setPen(discol);
    }

    int xm = steelItemFrame + checkcol + steelItemHMargin;

    QString s = mi->text();
    if(!s.isNull()) {
        int t = s.find('\t');
        int m = steelItemVMargin;
        const int text_flags = AlignVCenter|ShowPrefix|DontClip|SingleLine;
        if(t >= 0) {
            if(dis && !act) {
                p->setPen(g.light());
                p->drawText(x+w-tab-steelRightBorder-steelItemHMargin-steelItemFrame+1,
                             y+m+1, tab, h-2*m, text_flags, s.mid(t+1));
                p->setPen(discol);
            }
            p->drawText(x+w-tab-steelRightBorder-steelItemHMargin-steelItemFrame,
                         y+m, tab, h-2*m, text_flags, s.mid(t+1));
        }
        if(dis && !act) {
            p->setPen(g.light());
            p->drawText(x+xm+1, y+m+1, w-xm+1, h-2*m, text_flags, s, t);
            p->setPen(discol);
        }
        p->drawText(x+xm, y+m, w-xm-tab+1, h-2*m, text_flags, s, t);
    } else if(mi->pixmap()) {
        QPixmap *pixmap = mi->pixmap();
        if(pixmap->depth() == 1)
            p->setBackgroundMode(OpaqueMode);
        p->drawPixmap(x+xm, y+steelItemFrame, *pixmap);
        if(pixmap->depth() == 1)
            p->setBackgroundMode(TransparentMode);
    }
    if(mi->popup()) {
        int dim =(h-2*steelItemFrame) / 2;
        drawArrow(p, RightArrow, FALSE,
                  x+w - steelArrowHMargin - steelItemFrame - dim,  y+h/2-dim/2,
                  dim, dim, g, mi->isEnabled());
    }
}

void SteelStyle::drawHandle(QPainter *p, int x, int y, int w, int h)
{
    if(w > h) { ++x; --w; } else { ++y; --h; }
    if(w % 2) --w;
    if(h % 2) --h;
    p->setBrushOrigin(x+1, y+1);
    p->fillRect(x+1, y+1, w-2, h-2, *slbr);
}

void SteelStyle::drawKBarHandle(QPainter *p, int x, int y, int w, int h,
                                const QColorGroup &, KToolBarPos, QBrush*)
{
    drawHandle(p, x+2, y+2, w, h-4);
}

void SteelStyle::drawKMenuBar(QPainter *p, int x, int y, int w, int h,
                              const QColorGroup &g, bool, QBrush *)
{
    p->fillRect(x, y, w, h, g.brush(QColorGroup::Background));
}

void SteelStyle::drawKMenuItem(QPainter *p, int x, int y, int w, int h,
                               const QColorGroup &g, bool active, QMenuItem *mi, QBrush *)
{
    if(p->font() == KGlobalSettings::generalFont())
        p->setFont(KGlobalSettings::menuFont());

    if(active)
    {
        QBrush br = g.brush(QColorGroup::Highlight);
        qDrawShadePanel(p, x, y, w, h, g, true, 1, &br);
    }

    drawItem(p, x, y, w, h, AlignCenter|ShowPrefix|DontClip|SingleLine,
             g, mi->isEnabled(), mi->pixmap(), mi->text(), -1, &g.buttonText());
}

void SteelStyle::drawKToolBar(QPainter *p, int x, int y, int w, int h,
                              const QColorGroup &g, KToolBarPos, QBrush *)
{
    drawBevelButton(p, x, y, w, h, g, false, NULL);
}

void SteelStyle::drawKToolBarButton(QPainter *p, int x, int y, int w, int h,
                                    const QColorGroup &g, bool sunken,
                                    bool raised, bool enabled, bool popup,
                                    KToolButtonType icontext,
                                    const QString& btext, const QPixmap *pixmap,
                                    QFont *font, QWidget *)
{
    int dx, dy;

    QFontMetrics fm(*font);

    if(raised || sunken)
    {
        drawBevelButton(p, x, y, w, h, g, sunken, NULL);
    }
    else
    {
        p->fillRect(x, y, w, h, g.brush(QColorGroup::Background));
    }
    p->setPen(g.buttonText());

    if (icontext == Icon){ // icon only
        if (pixmap){
            dx = ( w - pixmap->width() ) / 2;
            dy = ( h - pixmap->height() ) / 2;
            if ( sunken )
            {
                ++dx;
                ++dy;
            }
            p->drawPixmap( x+dx, y+dy, *pixmap );
        }
    }
    else if (icontext == IconTextRight){ // icon and text (if any)
        if (pixmap){
            dx = 4;
            dy = ( h - pixmap->height() ) / 2;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            p->drawPixmap( x+dx, y+dy, *pixmap );
        }
        if (!btext.isNull()){
            int tf = AlignVCenter|AlignLeft;
            if (pixmap)
                dx= 4 + pixmap->width() + 2;
            else
                dx= 4;
            dy = 0;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            if (font)
                p->setFont(*font);
            if(raised)
                p->setPen(KGlobalSettings::toolBarHighlightColor());
            p->drawText(x+dx, y+dy, w-dx, h, tf, btext);
        }
    }
    else if (icontext == Text){ // only text, even if there is a icon
        if (!btext.isNull()){
            int tf = AlignVCenter|AlignLeft;
            if (!enabled)
                p->setPen(g.dark());
            dx= (w - fm.width(btext)) / 2;
            dy= (h - fm.lineSpacing()) / 2;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            if (font)
                p->setFont(*font);
            if(raised)
                p->setPen(KGlobalSettings::toolBarHighlightColor());
            p->drawText(x+dx, y+dy, fm.width(btext), fm.lineSpacing(), tf, btext);
        }
    }
    else if (icontext == IconTextBottom){
        if (pixmap){
            dx = (w - pixmap->width()) / 2;
            dy = (h - fm.lineSpacing() - pixmap->height()) / 2;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            p->drawPixmap( x+dx, y+dy, *pixmap );
        }
        if (!btext.isNull()){
            int tf = AlignBottom|AlignHCenter;
            dy= pixmap->height();
            dx = 2;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            if (font)
                p->setFont(*font);
            if(raised)
                p->setPen(KGlobalSettings::toolBarHighlightColor());
            p->drawText(x, y, w, h-3, tf, btext);
        }
    }
    if (popup){
        if (enabled)
            qDrawArrow (p, Qt::DownArrow, Qt::WindowsStyle, false, w-5, h-5, 0, 0,
                        g, true);
        else
            qDrawArrow (p, Qt::DownArrow, Qt::WindowsStyle, false, w-5, h-5,
                        0, 0, g, false);
    }
}

void SteelStyle::drawKickerAppletHandle(QPainter *p, int x, int y, int w, int h,
                                        const QColorGroup &, QBrush *)
{
    drawHandle(p, x, y, w, h);
}

#include "steelstyle.moc"
