#ifndef __STEEL_H__
#define __STEEL_H__

#include <kstyle.h>
#include <qpalette.h>

class SteelStyle: public KStyle
{
    Q_OBJECT
public:
    SteelStyle();
    QBrush *pmBrush(const QColor& bg, const QColor& br, const QColor& sh);
    void polish(QPalette& pal);
    void polish(QWidget *wdg);
    void unPolish(QWidget *wdg);
    void drawButton(QPainter *p, int x, int y, int w, int h,
		    const QColorGroup &g, bool sunken = false, const QBrush* fill = 0);
    void drawBevelButton(QPainter *p, int x, int y, int w, int h,
			 const QColorGroup &g, bool sunken = false, const QBrush *fill = 0);
    void drawPanel(QPainter *p, int x, int y, int w, int h,
		   const QColorGroup &g, bool sunken = false,
		   int lineWidth = 1, const QBrush *fill = 0);
    void drawPopupPanel(QPainter *p, int x, int y, int w, int h,
			const QColorGroup &g,
			int lineWidth = 1, const QBrush *fill = 0);
    void drawArrow(QPainter *p, Qt::ArrowType type, bool down,
		   int x, int y, int w, int h,
		   const QColorGroup &g, bool enabled, const QBrush *fill = 0);
    QSize exclusiveIndicatorSize() const;
    void drawExclusiveIndicatorMask(QPainter* p,  int x, int y, int w, int h, bool on);
    void drawExclusiveIndicator(QPainter* p,  int x, int y, int w, int h, const QColorGroup& g,
				bool on, bool down = false, bool enabled = true);
    QSize indicatorSize() const;
    void drawIndicator(QPainter* p,  int x, int y, int w, int h, const QColorGroup& g,
		       int state, bool down = false, bool ena = true);
    void drawFocusRect(QPainter *p, const QRect &r,
		       const QColorGroup &g, const QColor* bg = 0, bool = false);
    void drawComboButton(QPainter *p, int x, int y, int w, int h,
			 const QColorGroup &g, bool sunken = false,
			 bool editable = false, bool enabled = true, const QBrush *fill = 0);
    QRect comboButtonRect(int x, int y, int w, int h);
    QRect comboButtonFocusRect(int x, int y, int w, int h);
    void drawPushButton(QPushButton* btn, QPainter *p);
    void drawPushButtonLabel(QPushButton *btn, QPainter *p);
    void getButtonShift(int &x, int &y);
    int defaultFrameWidth() const;
    void tabbarMetrics(const QTabBar* t, int& hframe, int& vframe, int& overlap);
    void drawTab(QPainter* p, const QTabBar* tb, QTab* t, bool selected);
    void scrollBarMetrics(const QScrollBar* sb, int &sliderMin, int &sliderMax, int &sliderLength, int&buttonDim);
    void drawScrollBarControls(QPainter* p, const QScrollBar* sb,
			       int sliderStart, uint controls, uint activeControl);
    ScrollControl scrollBarPointOver(const QScrollBar* sb, int sliderStart, const QPoint& p);
    int sliderLength() const;
    void drawSlider(QPainter *p, int x, int y, int w, int h, const QColorGroup &g,
		    Orientation orient, bool tickAbove, bool tickBelow);
    void drawSliderGroove(QPainter *p, int x, int y, int w, int h,
			  const QColorGroup& g, QCOORD c, Orientation orient);
    int splitterWidth() const;
    void drawSplitter(QPainter *p, int x, int y, int w, int h,
		      const QColorGroup &g,  Orientation);
    void drawCheckMark(QPainter *p, int x, int y, int w, int h,
		       const QColorGroup &g, bool act, bool dis);
    void polishPopupMenu(QPopupMenu* p);
    int extraPopupMenuItemWidth(bool checkable, int maxpmw, QMenuItem* mi, const QFontMetrics&);
    int popupMenuItemHeight(bool, QMenuItem* mi, const QFontMetrics& fm);
    void drawPopupMenuItem(QPainter* p, bool checkable, int maxpmw, int tab, QMenuItem* mi,
			   const QPalette& pal,
			   bool act, bool enabled, int x, int y, int w, int h);
    void drawHandle(QPainter *p, int x, int y, int w, int h);
    void drawKBarHandle(QPainter *p, int x, int y, int w, int h,
         const QColorGroup &g, KToolBarPos, QBrush *);
    void drawKMenuBar(QPainter *p, int x, int y, int w, int h,
                      const QColorGroup &g, bool, QBrush *fill);
    void drawKMenuItem(QPainter *p, int x, int y, int w, int h,
                       const QColorGroup &g, bool active, QMenuItem *mi, QBrush *);
    void drawKToolBar(QPainter *p, int x, int y, int w, int h,
         const QColorGroup &g, KToolBarPos, QBrush *);
    void drawKToolBarButton(QPainter *p, int x, int y, int w, int h,
         const QColorGroup &g, bool sunken, bool raised, bool enabled, bool popup,
         KToolButtonType icontext, const QString& btext, const QPixmap *pixmap,
         QFont *font, QWidget *);
    void drawKickerAppletHandle(QPainter *p, int x, int y, int w, int h,
                                const QColorGroup &g, QBrush *);
private:
    QColor hl1, hl2, hl3;
    QBrush *sbbr, *slbr;
};

#endif
